dir="$(dirname $0)"
cd "$dir"

# Pre-cleanup
rm -rf cov/ $(stack path --dist-dir)/hpc/ ./stack-test-exe-2.tix ./stack-test-exe-1.tix ./all.tix
mkdir cov/

# Force a rebuild ; --force-dirty doesn't (as of 2017-08-07) do the
# right thing, at least by itself; see
# https://github.com/commercialhaskell/stack/issues/3306 and
# https://github.com/commercialhaskell/stack/issues/1940
for file in $(find app/ src/ test/ -type f)
do
  sed -i '$s/$/ /' $file
done

stack build --coverage --force-dirty stack-test
stack install --coverage --force-dirty stack-test
stack test --coverage --force-dirty stack-test

# Here we find the .tix files, sum *all* of them, put them where we
# want them, and clean up the originals.
#
# It's arguable that this step is stack's job, but if stack isn't
# going to do it but stack is going to generate the final markup,
# then there probably has to be some sort of hook to run a user
# script to do it, as "I need to run the executable in the test
# directory" is not an unreasonable requirement.
tixes=$(find . "$(stack path --local-hpc-root)/stack-test/" -name hpc -prune -o -type f -name '*.tix' -print | tr '\n' ' ')
if [ "$tixes" ]
then
  echo stack exec hpc -- sum --exclude=Main --union $tixes
  stack exec hpc -- sum --exclude=Main --union $tixes >all.tix

  /bin/rm -f ./.stack-work/dist/x86_64-linux/*/hpc/*
  /bin/rm -f ./.hpc/*
  /bin/rm -f $tixes
else
  echo "No tix files found at all; bailing."
  exit 1
fi

# Here we generate the markup.
echo stack exec hpc -- markup --hpcdir=hpc/ --destdir=cov/ --verbosity=2 all.tix
stack exec hpc -- markup --hpcdir=$(stack path --dist-dir)/hpc/ --destdir=cov/ --verbosity=2 all.tix

# Undo what we did to force a rebuild
for file in $(find app/ src/ test/ -type f)
do
  sed -i '$s/ *$//' $file
done
